@@ -2,7 +2,7 @@
 ;;;
 ;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2017, 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
-;;; Copyright © 2014, 2017, 2020, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2014, 2017, 2020, 2021, 2022, 2023, 2024 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2020, 2021, 2022, 2023 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
@@ -383,6 +383,7 @@
             "formal parameter ~a must not have the same name as its type"
             (.name o))))
        '())
+   (check-non-void o)
    (re-definition o)
    (let ((type (ast:type o))
          (event (ast:parent o <event>)))
@@ -424,11 +425,6 @@
              (append-map wfc (ast:function* o))
              (wfc (.statement o))))))))
 
-(define-method (wfc (o <variable>))
-  (append
-   (re-definition o)
-   (assign o)))
-
 (define-method (wfc-constraint (o <variable>) (port <port>))
   (append
    (if (feature-supported? "shared") '()
@@ -634,6 +630,7 @@
 
 (define-method (wfc (o <variable>))
   (append
+   (check-non-void o)
    (imperative-context o)
    (re-definition o)
    (assign o)))
@@ -692,6 +689,9 @@
 (define-method (wfc (o <reply>))
   (append
    (imperative-context o)
+   (let ((void? (compose (is? <void>) ast:type)))
+     (if (and=> (.expression o) (-> void? (is? <literal>))) '()
+         `(,(wfc-error o "cannot reply a non literal void expression"))))
    (if (.expression o) (wfc (.expression o)) '())
    (if (.port.name o) (reply-with-port o) (reply-without-port o))))
 
@@ -764,6 +764,9 @@
          (return-type (and (null? wfce) (ast:type o)))
          (last-statement-block (last-statement (outer-compound o))))
     (append wfce
+            (let ((void? (compose (is? <void>) ast:type)))
+              (if (and=> (.expression o) (-> void? (is? <literal>))) '()
+                  `(,(wfc-error o "cannot return a non literal void expression"))))
             (cond ((not function)
                    `(,(wfc-error o "cannot use return outside of function")))
                   ((pair? wfce) '())
@@ -1501,6 +1504,14 @@
     (and component
          (ast:lookup (.parent component) (.type.name o)))))
 
+(define (check-non-void o)
+  (if (not (as (ast:type o) <void>)) '()
+      (let ((type (if (is-a? o <formal>) "parameter"
+                      "variable"))
+            (name (.name o)))
+        (list (wfc-error o (format #f "~a `~a' cannot have type `void'"
+                                   type name))))))
+
 
 ;;;
 ;;; Entry points.
