@@ -361,7 +361,7 @@ pump::collateral_block (dzn::runtime &runtime, dzn::component *component)
   collateral_blocked.splice (collateral_blocked.end (), coroutines, self);
 
   self->component = component;
-  size_t coroutine_id = runtime.handling (component) | runtime.blocked (component);
+  size_t coroutine_id = runtime.handling (component) ? runtime.handling (component) : runtime.blocked (component);
   auto it = find_if (coroutines.begin (), coroutines.end (),
                      [coroutine_id] (dzn::coroutine & coroutine)
                      {return coroutine.id == coroutine_id;});
@@ -427,7 +427,8 @@ pump::block (dzn::runtime &runtime, dzn::component *component, void *port)
   auto self = find_self (coroutines);
   runtime.blocked (component) = self->id;
   runtime.handling (component) = 0;
-  runtime.flush (component, self->id);
+  runtime.flush (component, self->id, true);
+
   if (runtime.skip_block (component, port))
     {
       runtime.reset_skip_block (component);
